/*
 * Originally written by Alexander Vdolainen <avdolainen@gmail.com>
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <sys/un.h>
#include <errno.h>
#include <time.h>
#include <signal.h>
#include <getopt.h>
#include <syslog.h>
#include <pthread.h>
#include <sexpr/sexp.h>

#include <dsbus/data.h>
#include <dsbus/daemon.h>

#define MAX_PAYLOAD  512

static int __remove_object_storage(FILE *out, dsb_secstore_t *storage,
                                   uint64_t hash)
{
  int r = dsb_store_remove(storage, hash);

  /* NOTE: here the place to put the callback */

  return r;
}

void events_loop(FILE *out, struct dsbd_ctx *gctx, int fd)
{
  sexp_t *sx, *isx;
  int idx;
  uint64_t hash;
  dsb_secstore_t *storage = gctx->storage;
  char *datamsg, *cmd;
  struct nlmsghdr *nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD));
  struct iovec iov;
  struct msghdr msg;
  struct sockaddr_nl src_addr, dest_addr;

  for(;;) {
    memset(&src_addr, 0, sizeof(src_addr));
    src_addr.nl_family = AF_NETLINK;
    src_addr.nl_pid = getpid(); /* self pid */

    memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
    memset(&dest_addr, 0, sizeof(dest_addr));

    iov.iov_base = (void *)nlh;
    iov.iov_len = NLMSG_SPACE(MAX_PAYLOAD);
    msg.msg_name = (void *)&dest_addr;
    msg.msg_namelen = sizeof(dest_addr);
    msg.msg_iov = &iov;
    msg.msg_iovlen = 1;

    /* Read message from kernel */
    recvmsg(fd, &msg, 0);
    datamsg = NLMSG_DATA(nlh);

    if(strlen(datamsg) && (sx = parse_sexp(datamsg, strlen(datamsg)))) {
      SEXP_ITERATE_LIST(sx, isx, idx) {
        if(isx->ty == SEXP_LIST) goto __badprot;
        switch(idx) {
        case 0:
          cmd = isx->val;
          break;
        case 2:
          hash = strtoul(isx->val, NULL, 0);
          break;
        }
      }
      if(idx < 2) goto __badprot;

      if(!strcmp(cmd, "rm-object")) __remove_object_storage(out, storage, hash);

    __badprot:
      destroy_sexp(sx);
    }
  }
}

