/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * Originally written by Alexander Vdolainen <avdolainen@gmail.com>
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <sys/un.h>
#include <errno.h>
#include <time.h>
#include <signal.h>
#include <getopt.h>
#include <syslog.h>
#include <pthread.h>
#include <sexpr/sexp.h>

#include <dsbus/data.h>
#include <dsbus/daemon.h>

#define MAX_PAYLOAD  512

struct netlink_be {
  int fd;
};

int store_initdac(dsb_secstore_t *storage, void *priv)
{
  struct netlink_be *be = malloc(sizeof(struct netlink_be));
	struct sockaddr_nl src_addr;

  if(!be) return ENOMEM;
  else storage->dac_be = be;

	be->fd = socket(PF_NETLINK, SOCK_RAW, 31);
	if(be->fd < 0) return errno;

	memset(&src_addr, 0, sizeof(src_addr));
	src_addr.nl_family = AF_NETLINK;
	src_addr.nl_pid = getpid(); /* self pid */

	bind(be->fd, (struct sockaddr*)&src_addr, sizeof(src_addr));

  return 0;
}

int scknl_openevents(void)
{
	int fd;
	struct sockaddr_nl src_addr;

	fd = socket(PF_NETLINK, SOCK_RAW, 30);
	if(fd < 0) return fd;

	memset(&src_addr, 0, sizeof(src_addr));
	src_addr.nl_family = AF_NETLINK;
	src_addr.nl_pid = getpid(); /* self pid */

	bind(fd, (struct sockaddr*)&src_addr, sizeof(src_addr));

	return fd;
}

int netlink_senddsbmsg(dsb_secstore_t *storage, struct dsbnl_policy_msg *dsbmsg)
{
	struct netlink_be *be = storage->dac_be;
	struct sockaddr_nl dest_addr;
	struct nlmsghdr *nlh = NULL;
	struct msghdr msg;
	struct iovec iov;
	int r = 0;

	/* set destination address */
	memset(&dest_addr, 0, sizeof(dest_addr));
	dest_addr.nl_family = AF_NETLINK;
	dest_addr.nl_pid = 0; /* For Linux Kernel */
	dest_addr.nl_groups = 2; /* unicast */

	nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD));
	memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
	nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);
	nlh->nlmsg_pid = getpid();
	nlh->nlmsg_flags = 0;

	memcpy(NLMSG_DATA(nlh), dsbmsg, sizeof(struct dsbnl_policy_msg));

	iov.iov_base = (void *)nlh;
	iov.iov_len = nlh->nlmsg_len;
	memset(&msg, 0, sizeof(msg));
	msg.msg_name = (void *)&dest_addr;
	msg.msg_namelen = sizeof(dest_addr);
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;

	r = sendmsg(be->fd, &msg, 0);
	if(r < 0) {
		printf("go: %s\n", strerror(errno));
		return errno;
	}

	return 0;
}

int store_pushdac(dsb_secstore_t *storage, dsb_secinfo_t *info)
{
  /* still dummy */
  return 0;
}

