/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * Originally written by Alexander Vdolainen <avdolainen@gmail.com>
 */

#ifndef __DSBUS_DATA_H__
#define __DSBUS_DATA_H__

#include <stdint.h>
#include <sys/types.h>
#include <pthread.h>
#include <tdata/usrtc.h>

#define MAX_ACL_LIST_SIZE  32
#define __MXRLE  MAX_ACL_LIST_SIZE
#define MAX_CTRLED_OBJECTS  32768

#define DSB_AUDIT_EVENTS  (1 << 1)
#define DSB_CONFIGURED    (1 << 2)
#define DSB_GARBAGED      (1 << 3)
#define DSB_DIRTY         (1 << 4)

#define DSBACC_ALLOWHARDLINK  (1 << 12)

typedef struct __object_access_list_entry_type {
	union {
		uid_t uid;
		gid_t gid;
	};
  uint8_t mask;
} dsb_acl_entry_t;

typedef struct __object_access_list_type {
  dsb_acl_entry_t uidlist[__MXRLE];
	dsb_acl_entry_t gidlist[__MXRLE];
	uint8_t othmask;
} dsb_acl_t;

typedef struct __object_security_info_type {
  dsb_acl_t aclist;
  pthread_mutex_t slock;
  usrtc_node_t node;
  uint8_t flags;
  uint8_t blk_uuid[16]; /* 128bit uuid for a volume stored */
  uint64_t inode_num; /* inode number */
  char obj_name[256];
  uint64_t hash;
} dsb_secinfo_t;

typedef struct __object_security_store_type {
  usrtc_t curr_data;
  usrtc_t garbaged_data;
  pthread_rwlock_t srwlock;
  int (*store_initbe)(struct __object_security_store_type *, void *);
  int (*store_initdac)(struct __object_security_store_type *, void *);
  int (*store_pushbe)(struct __object_security_store_type *, dsb_secinfo_t *);
  int (*store_pushdac)(struct __object_security_store_type *, dsb_secinfo_t *);
  void *store_be;
  void *dac_be;
} dsb_secstore_t;

/* pointers */
#define dsb_sectore_setstoreptr(a, b)  (a)->store_be = (b)
#define dsb_sectore_setdacptr(a, b)  (a)->dac_be = (b)
#define dsb_sectore_storeptr(a)  (a)->store_be
#define dsb_sectore_dacptr(a)  (a)->dac_be
/* functions */
#define dsb_sectore_setinitdac(a, b)  (a)->store_initdac = (b)
#define dsb_sectore_setinitbe(a, b)  (a)->store_initbe = (b)
#define dsb_sectore_setpushdac(a, b)  (a)->store_pushdac = (b)
#define dsb_sectore_setpushbe(a, b)  (a)->store_pushbe = (b)
#define dsb_sectore_pushdac(a, b)  (a)->store_pushdac((a), (b))
#define dsb_sectore_pushbe(a, b)  (a)->store_pushbe((a), (b))

/* a little bit of API */
int dsb_store_allocinit(dsb_secstore_t *storage);
int dsb_store_init(dsb_secstore_t *storage, void *beptr, void *dacptr);
int dsb_store_flush(dsb_secstore_t *storage);
int dsb_store_clean(dsb_secstore_t *storage);
int dsb_store_insert(dsb_secstore_t *storage, dsb_secinfo_t *info);
int dsb_store_remove(dsb_secstore_t *storage, uint64_t hash);
int dsb_store_search(dsb_secstore_t *storage, uint64_t hash, dsb_secinfo_t **info);

int dsb_store_info_allocinit(dsb_secinfo_t *info);
int dsb_store_info_destroy(dsb_secinfo_t *info);

#endif /* __DSBUS_DATA_H__ */
